from pyglet.gl import *
from pyglet import image
from pyglet import media

import RCollisions
import RBullet

import data

from math import *
from constants import *

import random

class BadDude():
	def __init__(self, world, pos):
		self.world = world
		self.pos = pos
		#self.rotation = 0
		self.rect = RCollisions.Rect(self.pos, (self.texture.width, self.texture.height))
	
		glBindTexture(self.texture.target, self.texture.id)
		glTexParameterf(self.texture.target, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		glTexParameterf(self.texture.target, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameterf(self.texture.target, GL_TEXTURE_WRAP_S, GL_CLAMP)
		glTexParameterf(self.texture.target, GL_TEXTURE_WRAP_T, GL_CLAMP)	
	
	def hit(self, damage):
		self.health -= damage
	
	def update(self):
		self.pos[1] -= self.speed - self.world.speed
		self.rect = RCollisions.Rect(self.pos, (self.texture.width, self.texture.height))
		
		for gun in self.guns:
			gun.update()
		
		if self.health <= 0:
			sound = media.load(data.filepath("explosion.wav"), streaming=False)
			sound.play()		
			return BADDUDE_KILLED
		if self.pos[1] + self.texture.height < 0:
			return BADDUDE_ESCAPED
		return False
		
	def draw(self):
		glPushMatrix()
		glTranslatef(self.pos[0], self.pos[1], 0)		
		#glRotatef(self.rotation, 0.0, 0.0, 1.0)	
		self.texture.blit(-self.texture.width / 2, -self.texture.height / 2)
		glPopMatrix()
		for gun in self.guns:
			gun.draw()
		
class Gun:
	def __init__(self, world, turretTexture, parentPos, pos, direction, damage, rate, fixed):
		self.world = world
		self.pos = pos
		self.parentPos = parentPos
		self.direction = direction
		self.fixed = fixed
		self.reload = rate
		self.rate = rate
		self.damage = damage
		self.gunRotation = 0
		self.gunRotationRads = 0	
		self.turretTexture = turretTexture
				
		self.bullets = []
		
	def shoot(self):
		if self.reload == 0:
			self.reload = self.rate
			self.bullets.append(RBullet.Bullet(self.world.robots, 100, self.damage, (self.parentPos[0] + self.pos[0], self.parentPos[1] + self.pos[1]), (GUN_SPEED * cos(self.direction + self.gunRotationRads), GUN_SPEED * sin(self.direction + self.gunRotationRads))))
		#	print self.parentPos[0] + self.pos[0], self.parentPos[1] + self.pos[1]
	
	def update(self):
		if self.parentPos[1] > 600:
			return
		if not self.fixed:
			maxRange = (100 * GUN_SPEED) ** 2
			minRange = (100 * GUN_SPEED) ** 2
			target = None
			spareTargets = []
			for robot in self.world.robots:
				rng = (self.parentPos[0] + self.pos[0] - robot.pos[0]) ** 2 + (self.parentPos[1] + self.pos[1] - robot.pos[1]) ** 2
				if (rng < minRange):
					minRange = rng
					target = robot
				if rng < maxRange:
					spareTargets.append(robot)
			
			# Found target. Shoot the bugger
			if target:
				gunRequiredAngle = atan2(target.pos[1] - self.parentPos[1] - self.pos[1], target.pos[0] - self.parentPos[0] - self.pos[0]) + pi / 2
								
				rotation = self.gunRotationRads - gunRequiredAngle
								
				# Very hacky. 
				while rotation > pi:
					rotation -= 2 * pi
				while rotation < -pi:
					rotation += 2 * pi
											
				if rotation > 0:
					self.gunRotationRads += max(-rotation, -0.1)
				else:
					self.gunRotationRads += min(-rotation, 0.1)
					
				while self.gunRotationRads > pi:
					self.gunRotationRads -= 2 * pi
				while self.gunRotationRads < -pi:
					self.gunRotationRads += 2 * pi
											
				self.gunRotation = self.gunRotationRads * 180 / pi
				
				bShootAnyway = False
				
				for robot in spareTargets:
					if robot != target:
						spareRot = self.gunRotationRads + atan2(robot.pos[1] - self.parentPos[1] - self.pos[1], robot.pos[0] - self.parentPos[0] - self.pos[0]) + pi / 2
						while spareRot > pi:
							spareRot -= 2 * pi
						while spareRot < -pi:
							spareRot += 2 * pi					
						if fabs(spareRot) < 0.1:
							bShootAnyway = True
							break
						
				
				if bShootAnyway or fabs(rotation) < 0.1:
					self.shoot()
		else:
			self.shoot()
				
		delList = []
		for bullet in self.bullets:
			if bullet.update():
				delList.append(bullet)
		for bullet in delList:
			self.bullets.remove(bullet)
		self.reload -= 1
		
	def draw(self):
		for bullet in self.bullets:
			bullet.draw()
		if self.turretTexture:
			glPushMatrix()
			glTranslatef(self.parentPos[0] + self.pos[0], self.parentPos[1] + self.pos[1], 0)	
			glRotatef(self.gunRotation, 0.0, 0.0, 1.0)	
			self.turretTexture.blit(-self.turretTexture.width / 2, -self.turretTexture.height / 2)
			glPopMatrix()			
		
class Tank(BadDude):
	def __init__(self, world, pos, wave):
		pos = list(pos)
		self.health = 100 * 1.1 ** (wave / NUM_BADDUDES)
		self.speed = 2 * 1.02 ** (wave / NUM_BADDUDES)
		self.damage = 5 * 1.1 ** (wave / NUM_BADDUDES)
		self.texture = image.load(data.filepath("tank1.png")).texture
		self.turretTexture = image.load(data.filepath("tank1turret.png")).texture
		self.guns = [Gun(world, self.turretTexture, pos, (0, -5), -pi / 2, self.damage, 50, False)]
		BadDude.__init__(self, world, pos)	
		
		
class Fighter(BadDude):
	def __init__(self, world, pos, wave):
		pos = list(pos)
		self.world = world
		self.health = 40 * 1.1 ** (wave / NUM_BADDUDES)
		self.speed = 3 * 1.02 ** (wave / NUM_BADDUDES)
		self.damage = 5 * 1.1 ** (wave / NUM_BADDUDES)
		self.texture = image.load(data.filepath("fig1.png")).texture
		self.turretTexture = None
		self.guns = (Gun(world, self.turretTexture, pos, (-21, -4), -pi / 2, self.damage, 60, True), 
					 Gun(world, self.turretTexture, pos, (20, -4), -pi / 2, self.damage, 60, True),
					 Gun(world, self.turretTexture, pos, (-15, -11), -pi / 2, self.damage, 60, True),
					 Gun(world, self.turretTexture, pos, (14, -11), -pi / 2, self.damage, 60, True))
		BadDude.__init__(self, world, pos)
				

		