from math import *
from constants import *

class Rect:
	def __init__(self, *args):
		if len(args) == 2:
			centre, size = args
			self.left = centre[0] - size[0] / 2
			self.right = centre[0] + size[0] / 2
			self.top = centre[1] + size[1] / 2
			self.bottom = centre[1] - size[1] / 2
			
			self.bottomLeft = (self.left, self.bottom)
			self.topLeft = (self.left, self.top)
			self.bottomRight = (self.right, self.bottom)
			self.topRight = (self.right, self.top)
		else:
			self.left, self.bottom, width, height = args
			
			self.right = self.left + width
			self.top = self.bottom + height
	
def pointInRect(point, rect):
	if point[1] < rect.bottom or point[0] < rect.left or point[0] > rect.right or point[1] > rect.top:
		return False
	return True
	
def dot(vec1, vec2):
	return vec1[0] * vec2[0] + vec1[1] * vec2[1]
	
def minmax(*values):
	return min(values), max(values)
	
def getRectCorners(centre, dimensions, rotation):
	# From the bottom left and the rotation we know the position of the other 4 points.
	# Super
	# If this gets slow fixed bodies can skip this calculation every time.
	bottomLeft = - dimensions[0] / 2, - dimensions[1] / 2
	
	r, t = sqrt(bottomLeft[0] ** 2 + bottomLeft[1] ** 2), atan2(bottomLeft[1], bottomLeft[0])
	
	t += rotation
	
	x1, y1 = r * cos(t) + centre[0], r * sin(t) + centre[1]
	
	cosRot = cos(rotation)
	sinRot = sin(rotation)
	
	x2, y2 = x1 + dimensions[0] * cosRot, y1 + dimensions[0] * sinRot
	x3, y3 = x2 + dimensions[1] * (-sinRot), y2 + dimensions[1] * cosRot	
	x4, y4 = x1 + dimensions[1] * (-sinRot), y1 + dimensions[1] * cosRot
		
	return (x1, y1), (x2, y2), (x3, y3), (x4, y4)
	
from pyglet.gl import *
	
def rectCollide(centre1, dimensions1, rotation1, centre2, dimensions2, rotation2):
	
	distance = (centre1[0] - centre2[0]) ** 2 + (centre1[1] - centre2[1]) ** 2
	circle = (max(dimensions1) + max(dimensions2)) ** 2
	
	if distance > circle:
		return False

	cornerA = getRectCorners(centre1, dimensions1, rotation1)
	cornerB = getRectCorners(centre2, dimensions2, rotation2)
	
	#print centre1, centre2
	#	
	#print cornerA
	#print cornerB
	
	# if not RELEASE:
		# glColor4f(0, 1, 0, 0.5)
		# glBegin(GL_QUADS)
		# glVertex2f(cornerA[0][0] - centre1[0], cornerA[0][1] - centre1[1] + 300)
		# glVertex2f(cornerA[1][0] - centre1[0], cornerA[1][1] - centre1[1] + 300)
		# glVertex2f(cornerA[2][0] - centre1[0], cornerA[2][1] - centre1[1] + 300)
		# glVertex2f(cornerA[3][0] - centre1[0], cornerA[3][1] - centre1[1] + 300)
		# glEnd()
		# glColor4f(0, 0, 1, 0.5)
		# glBegin(GL_QUADS)
		# glVertex2f(cornerB[0][0] - centre1[0] + 400, cornerB[0][1] - centre1[1] + 300)
		# glVertex2f(cornerB[1][0] - centre1[0] + 400, cornerB[1][1] - centre1[1] + 300)
		# glVertex2f(cornerB[2][0] - centre1[0] + 400, cornerB[2][1] - centre1[1] + 300)
		# glVertex2f(cornerB[3][0] - centre1[0] + 400, cornerB[3][1] - centre1[1] + 300)
		# glEnd()	
		             
	for i in range(4):
		if i == 0:
			vec = cornerA[0][0] - cornerA[1][0], cornerA[0][1] - cornerA[1][1]
		elif i == 1:
			vec = cornerB[0][0] - cornerB[1][0], cornerB[0][1] - cornerB[1][1]
		elif i == 2:
			vec = cornerA[0][1] - cornerA[1][1], -(cornerA[0][0] - cornerA[1][0])
		elif i == 3:
			vec = cornerB[0][1] - cornerB[1][1], -(cornerB[0][0] - cornerB[1][0])
			
		#print vec	
			
		a = minmax(dot(cornerA[0], vec), dot(cornerA[1], vec), dot(cornerA[2], vec), dot(cornerA[3], vec))
		b = minmax(dot(cornerB[0], vec), dot(cornerB[1], vec), dot(cornerB[2], vec), dot(cornerB[3], vec))
			
		#print a, b	
			
		if (a[0] > b[1] or a[1] < b[0]):
			return False
			

	return True
	