#from pyglet.gl import *
#from pyglet import image
#from OpenGL.GL import *
from pyglet.gl import *
from pyglet import image
from pyglet import media

import data

import RSmoke
import RCollisions
import RBullet

from math import *
from constants import *
import random

class Player:
	def __init__(self, world, pos):
		self.world = world
		self.bodyTex = image.load(data.filepath("robot.png")).texture
		self.gunTex = image.load(data.filepath("robotgun.png")).texture
		leftTrack = image.load(data.filepath("leftTrack.png")).texture
		rightTrack = image.load(data.filepath("rightTrack.png")).texture
		self.leftTrackTex = [leftTrack.get_region(i, 0, leftTrack.width - 12, leftTrack.height) for i in range(12)]
		self.rightTrackTex = [rightTrack.get_region(i, 0, rightTrack.width - 12, rightTrack.height) for i in range(12)]
		
		for id in self.bodyTex.id, leftTrack.id, rightTrack.id:
			glBindTexture(GL_TEXTURE_2D, id)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
		
		self.leftIndex = 0
		self.rightIndex = 0
		self.rotation = -90
		self.rotationRads = -pi / 2
		self.gunRotation = -180	
		self.gunRotationRads = -pi
		self.pos = list(pos)
		self.smokes = []
		self.iTick = 0
				
		self.smokeStackPos = ((sqrt(27 ** 2 + 19 ** 2) / 2, atan2(19, -27)), (sqrt(27 ** 2 + 19 ** 2) / 2, atan2(-19, -27)))
		self.gunMuzzlePos = ((sqrt(20 ** 2 + 9 ** 2) / 2, atan2(9, 20)), (sqrt(20 ** 2 + 9 ** 2) / 2, atan2(-9, 20)))
		
		self.leftReload = 0
		self.rightReload = 0
		
		self.gunLife = GUN_LIFE
		self.gunDamage = BULLET_DAMAGE
		self.health = 500
		self.maxHealth = 500
		self.speed = WORLD_SPEED
		
		self.healthLevel = 0
		self.rangeLevel = 0
		self.speedLevel = 0
		self.gunLevel = 0
		
		self.bReverse = False
		self.bSpawning = True
		
		self.rect = RCollisions.Rect(self.pos, (self.bodyTex.width, self.bodyTex.height))
			
	def canGunUp(self):
		return self.gunLife
		
	def healthUp(self):
		self.healthLevel += 1
		self.health += 500
		self.maxHealth += 500
	
	def gunUp(self):
		self.gunLevel += 1
		self.gunDamage += (BULLET_DAMAGE / 5) << self.gunLevel
		print self.gunLife
		self.gunLife = max(0, self.gunLife - GUN_LIFE / 5)
		print self.gunLife
	
	def speedUp(self):
		self.speedLevel += 1
		self.speed += WORLD_SPEED / 5
	
	def rangeUp(self):
		self.rangeLevel += 1
		self.gunLife += GUN_LIFE / 5
		
	def checkForCollisions(self):
		for robot in self.world.robots:
			if robot != self:
				if RCollisions.rectCollide(self.pos, (self.bodyTex.width / 2 + 5, self.bodyTex.height / 2 + 5), self.rotationRads, robot.pos, (robot.bodyTex.width / 2 + 5, robot.bodyTex.height / 2 + 5), robot.rotationRads):
					return True
		return False
	
	def forward(self, bCollisions = True):
		self.leftIndex -= 2
		self.rightIndex -= 2
		self.pos[0] += cos(self.rotationRads) * self.speed
		self.pos[1] += sin(self.rotationRads) * self.speed
		if bCollisions and self.checkForCollisions():
			self.backward(False)
		
	def backward(self, bCollisions = True):
		self.leftIndex += 2
		self.rightIndex += 2
		self.pos[0] -= cos(self.rotationRads) * self.speed
		self.pos[1] -= sin(self.rotationRads) * self.speed
		if bCollisions and self.checkForCollisions():
			self.forward(False)
		
	def left(self, bCollisions = True):
		self.leftIndex -= 2
		self.rotation += 180 / (50 * pi)
		self.rotationRads = self.rotation * pi / 180
		
		if bCollisions and self.checkForCollisions():
			self.right(False)		

	def right(self, bCollisions = True):
		self.rightIndex -= 2
		self.rotation -= 180 / (50 * pi)
		self.rotationRads = self.rotation * pi / 180
		
		if bCollisions and self.checkForCollisions():
			self.left(False)			
	
	def hit(self, damage):
		self.health -= damage
	
	def update(self, bLeft, bRight, bBack):
		if self.health <= 0:
			return True
	
		self.iTick += 1
		
		#TODO: Forwards and backwards
		if bLeft:
			self.left()
		elif bRight:
			self.right()
		else:
			if self.rotation < 0 or self.rotation > 180:
				self.bReverse = True
			else:
				self.bReverse = False
				
			if self.bReverse:
				if self.rotation > 180:
					self.rotation += min(270 - self.rotation, 180 / (50 * pi))
					self.leftIndex -= 1
				elif self.rotation > -90:
					self.rotation -= min(90 + self.rotation, 180 / (50 * pi))	
					self.rightIndex -= 1
			else:
				if self.rotation < 90:
					self.rotation += min(90 - self.rotation, 180 / (50 * pi))
					self.leftIndex -= 1
				elif self.rotation > 90:
					self.rotation -= min(self.rotation - 90, 180 / (50 * pi))
					self.rightIndex -= 1
				
			self.rotationRads = self.rotation * pi / 180
				
		if not self.bSpawning:
			if self.bReverse:
				if bBack:
					self.backward()
				self.pos[0] += cos(self.rotationRads) * self.speed
				self.pos[1] += sin(self.rotationRads) * self.speed
				if self.checkForCollisions():
					self.pos[0] -= cos(self.rotationRads) * self.speed
					self.pos[1] -= sin(self.rotationRads) * self.speed					
			else:
				if bBack:
					self.forward()			
				self.pos[0] -= cos(self.rotationRads) * self.speed
				self.pos[1] -= sin(self.rotationRads) * self.speed
				if self.checkForCollisions():
					self.pos[0] += cos(self.rotationRads) * self.speed
					self.pos[1] += sin(self.rotationRads) * self.speed					
		
		if self.pos[0] > 700:
			self.pos[0] = 700
		elif self.pos[0] < 50:
			self.pos[0] = 50
		if self.pos[1] > 550:
			self.pos[1] = 550
		
		self.pos[1] += self.world.speed
		
		if self.pos[1] >= 100:
			self.bSpawning = False
		
		if not self.bSpawning:
			if self.pos[1] < 100:
				self.pos[1] = 100
				if self.bReverse:
					self.leftIndex -= 2 * self.world.speed
					self.rightIndex -= 2 * self.world.speed				
				else:
					self.leftIndex += 2 * self.world.speed
					self.rightIndex += 2 * self.world.speed			
			else:
				if self.bReverse:
					self.leftIndex -= 2 * self.speed
					self.rightIndex -= 2 * self.speed				
				else:
					self.leftIndex += 2 * self.speed
					self.rightIndex += 2 * self.speed					

				
		self.rect = RCollisions.Rect(self.pos, (self.bodyTex.width, self.bodyTex.height))
				
	
				
		while self.rotation > 270:
			self.rotation -= 360
		while self.rotation < -90:
			self.rotation += 360						

		delList = []
		for smoke in self.smokes:
			if smoke.update():
				delList.append(smoke)
		for smoke in delList:
			self.smokes.remove(smoke)
		if not self.iTick % 10:		
			self.smokes.append(RSmoke.Smoke(self.world,
											(self.pos[0] + self.smokeStackPos[0][0] * cos(self.rotationRads + self.smokeStackPos[0][1]),
											 self.pos[1] + self.smokeStackPos[0][0] * sin(self.rotationRads + self.smokeStackPos[0][1]))))
		if not (self.iTick + 5) % 10:		
			self.smokes.append(RSmoke.Smoke(self.world,
											(self.pos[0] + self.smokeStackPos[1][0] * cos(self.rotationRads + self.smokeStackPos[1][1]),
											 self.pos[1] + self.smokeStackPos[1][0] * sin(self.rotationRads + self.smokeStackPos[1][1]))))
		
		
		# Gun seek target
		maxRange = (self.gunLife * GUN_SPEED) ** 2
		minRange = (self.gunLife * GUN_SPEED) ** 2
		target = None
		spareTargets = []
		for badDude in self.world.badDudes:
			if badDude.pos[1] < 700:
				rng = (self.pos[0] - badDude.pos[0]) ** 2 + (self.pos[1] - badDude.pos[1]) ** 2
				if (rng < minRange):
					minRange = rng
					target = badDude
				if rng < maxRange:
					spareTargets.append(badDude)
			
		# Found target. Shoot the bugger
		if target:
			gunRequiredAngle = atan2(target.pos[1] - self.pos[1], target.pos[0] - self.pos[0])
			
			currentRotation = self.gunRotationRads + self.rotationRads
			
			rotation = currentRotation - gunRequiredAngle
							
			# Very hacky. 
			while rotation > pi:
				rotation -= 2 * pi
			while rotation < -pi:
				rotation += 2 * pi
										
			if rotation > 0:
				self.gunRotationRads += max(-rotation, -0.1)
			else:
				self.gunRotationRads += min(-rotation, 0.1)
				
			while self.gunRotationRads > pi:
				self.gunRotationRads -= 2 * pi
			while self.gunRotationRads < -pi:
				self.gunRotationRads += 2 * pi
										
			self.gunRotation = self.gunRotationRads * 180 / pi
			
			bShootAnyway = False
			
			for badDude in spareTargets:
				if badDude != target:
					if badDude.pos[1] < 700:
						spareRot = self.gunRotationRads + self.rotationRads - atan2(badDude.pos[1] - self.pos[1], badDude.pos[0] - self.pos[0])
						while spareRot > pi:
							spareRot -= 2 * pi
						while spareRot < -pi:
							spareRot += 2 * pi					
						if fabs(spareRot) < 0.1:
							bShootAnyway = True
							break
					
			
			if bShootAnyway or fabs(rotation) < 0.1:
				if random.random() > 0.5:
					mainReload = self.leftReload
					secondaryReload = self.rightReload
					bLeft = True
				else:
					mainReload = self.rightReload
					secondaryReload = self.leftReload
					bLeft = False
					
				if mainReload == 0:
					sound = media.load(data.filepath("pop-SodaBush-1015.wav"), streaming=False)
					sound.play()
					if bLeft:
						self.world.bullets.append(RBullet.Bullet(self.world.badDudes, self.gunLife, self.gunDamage,
																 (self.pos[0] + self.gunMuzzlePos[0][0] * cos(self.rotationRads + self.gunMuzzlePos[0][1] + self.gunRotationRads),
																 self.pos[1] + self.gunMuzzlePos[0][0] * sin(self.rotationRads + self.gunMuzzlePos[0][1] + self.gunRotationRads)),				
																 (GUN_SPEED * cos(self.rotationRads + self.gunRotationRads), GUN_SPEED * sin(self.rotationRads + self.gunRotationRads))))
						self.leftReload += 10
					else:
						self.world.bullets.append(RBullet.Bullet(self.world.badDudes, self.gunLife, self.gunDamage,
																 (self.pos[0] + self.gunMuzzlePos[1][0] * cos(self.rotationRads + self.gunMuzzlePos[1][1] + self.gunRotationRads),
																 self.pos[1] + self.gunMuzzlePos[1][0] * sin(self.rotationRads + self.gunMuzzlePos[1][1] + self.gunRotationRads)),				
																 (GUN_SPEED * cos(self.rotationRads + self.gunRotationRads), GUN_SPEED * sin(self.rotationRads + self.gunRotationRads))))
						self.rightReload += 10
				elif mainReload < 8 and secondaryReload == 0:
					sound = media.load(data.filepath("pop-SodaBush-1015.wav"), streaming=False)
					sound.play()				
					if bLeft:
						self.world.bullets.append(RBullet.Bullet(self.world.badDudes, self.gunLife, self.gunDamage,
																 (self.pos[0] + self.gunMuzzlePos[1][0] * cos(self.rotationRads + self.gunMuzzlePos[1][1] + self.gunRotationRads),
																 self.pos[1] + self.gunMuzzlePos[1][0] * sin(self.rotationRads + self.gunMuzzlePos[1][1] + self.gunRotationRads)),				
																 (GUN_SPEED * cos(self.rotationRads + self.gunRotationRads), GUN_SPEED * sin(self.rotationRads + self.gunRotationRads))))
						self.rightReload += 10
					else:
						self.world.bullets.append(RBullet.Bullet(self.world.badDudes, self.gunLife, self.gunDamage,
																 (self.pos[0] + self.gunMuzzlePos[0][0] * cos(self.rotationRads + self.gunMuzzlePos[0][1] + self.gunRotationRads),
																 self.pos[1] + self.gunMuzzlePos[0][0] * sin(self.rotationRads + self.gunMuzzlePos[0][1] + self.gunRotationRads)),				
																 (GUN_SPEED * cos(self.rotationRads + self.gunRotationRads), GUN_SPEED * sin(self.rotationRads + self.gunRotationRads))))
						self.leftReload += 10
				
		self.leftReload = max(0, self.leftReload - 1)
		self.rightReload = max(0, self.rightReload - 1)
		
		return False
				
	def draw(self, selected):
	
		glPushMatrix()
		glTranslatef(self.pos[0], self.pos[1], 0)		
		glRotatef(self.rotation, 0.0, 0.0, 1.0)
		
		glScalef(0.5, 0.5, 1.0)
		self.leftTrackTex[(int(self.leftIndex) % 24) / 2].blit(-self.leftTrackTex[0].width / 2 - 2,  - self.leftTrackTex[0].height / 2)
		self.rightTrackTex[(int(self.rightIndex) % 24) / 2].blit(-self.rightTrackTex[0].width / 2 - 2, -self.rightTrackTex[0].height / 2)

		self.bodyTex.blit(-self.bodyTex.width / 2, -self.bodyTex.height / 2)
		
		if selected:
			glColor4f(1.0, 0.0, 0.0, 0.7)
			glBegin(GL_LINE_LOOP)
			glVertex2f(-self.bodyTex.width / 2 - 10, -self.bodyTex.height / 2 - 10)
			glVertex2f(self.bodyTex.width / 2 + 10, -self.bodyTex.height / 2 - 10)
			glVertex2f(self.bodyTex.width / 2 + 10, self.bodyTex.height / 2 + 10)
			glVertex2f(-self.bodyTex.width / 2 - 10, self.bodyTex.height / 2 + 10)
			glEnd()
		
			radius = 2 * self.gunLife * GUN_SPEED	

			glColor4f(0.0, 0.0, 1.0, 0.7)
			glBegin(GL_LINE_LOOP)
			for i in range(30):
				rad = i * pi / 15
				glVertex2f(cos(rad) * radius, sin(rad) * radius)
			glEnd()
					
		
		glRotatef(self.gunRotation, 0.0, 0.0, 1.0)
		self.gunTex.blit(-self.bodyTex.width / 2, -self.bodyTex.height / 2)
			
		glPopMatrix()
		
		for smoke in self.smokes:
			smoke.draw()
			
		glPushMatrix()
		glTranslatef(self.pos[0], self.pos[1], 0)				
		# Health bar
		healthRatio = float(self.health) / float(self.maxHealth)
		widthRadio = ((self.bodyTex.width / 2) - 20) * healthRatio
		glBegin(GL_QUADS)
		glColor4f(0.0, 1.0, 0.0, 0.9)
		glVertex2f(-self.bodyTex.width / 4 + 10, self.bodyTex.height / 4)
		glVertex2f(-self.bodyTex.width / 4 + 10, self.bodyTex.height / 4 + 4)
		glVertex2f(-self.bodyTex.width / 4 + 10 + widthRadio, self.bodyTex.height / 4 + 4)
		glVertex2f(-self.bodyTex.width / 4 + 10 + widthRadio, self.bodyTex.height / 4)
		glColor4f(1.0, 0.0, 0.0, 0.9)
		glVertex2f(-self.bodyTex.width / 4 + 10 + widthRadio, self.bodyTex.height / 4 + 4)
		glVertex2f(-self.bodyTex.width / 4 + 10 + widthRadio, self.bodyTex.height / 4)
		glVertex2f(self.bodyTex.width / 4 - 10, self.bodyTex.height / 4)
		glVertex2f(self.bodyTex.width / 4 - 10 , self.bodyTex.height / 4 + 4)		
		glEnd()
		glPopMatrix()