import RPlayer
import RCollisions
import RBadDudes

from pyglet.gl import *
from pyglet.window import key
from pyglet.window import mouse
from pyglet import font
from pyglet import image

import random
from math import *
from constants import *

import sys

import data

class World:
	def __init__(self):
		self.robots = [RPlayer.Player(self, (200, -64)), RPlayer.Player(self, (600, -64))]
		self.selectedRobots = []
		self.groundTex = image.load(data.filepath("grass.png")).texture
		self.plusTex = image.load(data.filepath("plus.png")).texture
		self.minusTex = image.load(data.filepath("minus.png")).texture
		self.speedBarTex = image.load(data.filepath("speedbar.png")).texture
		self.rangeUpTex = image.load(data.filepath("rangeup.png")).texture
		self.gunUpTex = image.load(data.filepath("gunup.png")).texture
		self.speedUpTex = image.load(data.filepath("speedup.png")).texture		
		self.healthUpTex = image.load(data.filepath("healthup.png")).texture		
		self.robotUpTex = image.load(data.filepath("robotup.png")).texture		
		self.topPanelTex = image.load(data.filepath("toppanel.png")).texture		
		self.goldTex = image.load(data.filepath("gold.png")).texture		
		self.lifeTex = image.load(data.filepath("life.png")).texture		
		self.disabledButtonTex = image.load(data.filepath("disabledbutton.png")).texture		
		
		self.plusRect = RCollisions.Rect(800 - 56, 600 - 35, 32, 32)
		self.minusRect = RCollisions.Rect(800 - 56, 600 - 319, 32, 32)
		self.speedBarRect = RCollisions.Rect(800 - 56, 600 - 290, 32, 256)		
		self.rangeUpRect = RCollisions.Rect(800 - 72, 600 - 360, 64, 32)
		self.gunUpRect = RCollisions.Rect(800 - 72, 600 - 415, 64, 32)
		self.speedUpRect = RCollisions.Rect(800 - 72, 600 - 470, 64, 32)
		self.healthUpRect = RCollisions.Rect(800 - 72, 600 - 525, 64, 32)
		self.robotUpRect = RCollisions.Rect(800 - 72, 600 - 580, 64, 32)
		
		font.add_file(data.filepath('MAKISUPA.TTF'))
		self.font = font.load('MAKISUPA.TTF', 13, dpi=96, bold = True)
		
				
		self.bullets = []
		self.badDudes = []
		self.gold = 0
		self.score = 0
		self.lives = 20
		self.waves = 0 
		
		self.scrollY = 0
		self.ticksUntilNextWave = TICKS_BETWEEN_WAVES / 4
		self.distanceMoved = 0
		
		self.speed = 1.0
		self.maxSpeed = 1.0	
		
	def getSpeedCost(self):
		cost = 0
		for robot in self.selectedRobots:
			cost += 10 << robot.speedLevel
		return cost
	def getRangeCost(self):
		cost = 0
		for robot in self.selectedRobots:
			cost += 10 << robot.rangeLevel
		return cost
	def getGunCost(self):
		cost = 0
		for robot in self.selectedRobots:
			cost += 10 << robot.gunLevel
		return cost
	def getHealthCost(self):
		cost = 0
		for robot in self.selectedRobots:
			cost += 10 << robot.healthLevel
		return cost	
	def getRobotCost(self):
		cost = 250
		maxSpeed = sys.maxint
		for robot in self.robots:
			if robot.speedLevel < maxSpeed:
				maxSpeed = robot.speedLevel
		if maxSpeed:
			cost += 10 << (maxSpeed - 1)
		return cost	

	def minus(self):
		self.speed = max(self.speed - self.maxSpeed / 10.0, 0)
	def plus(self):
		self.speed = min(self.speed + self.maxSpeed / 10.0, self.maxSpeed)
	def gun(self):
		cost = self.getGunCost()
		for robot in self.selectedRobots:
			if not robot.canGunUp():
				return True
		if cost <= self.gold:
			for robot in self.selectedRobots:
				robot.gunUp()	
			self.gold -= cost	
	def range(self):
		cost = self.getRangeCost()
		if cost <= self.gold:
			for robot in self.selectedRobots:
				robot.rangeUp()
			self.gold -= cost	
	def healthUp(self):
		cost = self.getHealthCost()
		if cost <= self.gold:
			for robot in self.selectedRobots:
				robot.healthUp()	
			self.gold -= cost	
	def speedUp(self):
		cost = self.getSpeedCost()
		if cost <= self.gold:
			for robot in self.selectedRobots:
				robot.speedUp()
				maxSpeed = sys.maxint
				for robot in self.robots:
					if robot.speed < maxSpeed:
						maxSpeed = robot.speed
					
			speedRatio = self.speed / self.maxSpeed
						
			self.maxSpeed = maxSpeed
			
			self.speed = speedRatio * maxSpeed
				
			self.gold -= cost	
	def robot(self):
		cost = self.getRobotCost()
		if cost <= self.gold:
			maxSpeed = sys.maxint
			for robot in self.robots:
				if robot.bSpawning:
					return True
				if robot.speedLevel < maxSpeed:
					maxSpeed = robot.speedLevel
			self.robots.append(RPlayer.Player(self, (400, -64)))
			for i in range(maxSpeed):
				self.robots[-1].speedUp()
			self.gold -= cost		
		
	def click(self, x, y, button, modifiers):
		if RCollisions.pointInRect((x, y), self.plusRect):
			self.plus()
			return True
		elif RCollisions.pointInRect((x, y), self.minusRect):
			self.minus()
			return True
		elif RCollisions.pointInRect((x, y), self.rangeUpRect):
			self.range()
			return True
		elif RCollisions.pointInRect((x, y), self.gunUpRect):
			self.gun()
			return True
		elif RCollisions.pointInRect((x, y), self.healthUpRect):
			self.healthUp()
			return True
		elif RCollisions.pointInRect((x, y), self.speedUpRect):	
			self.speedUp()
			return True
		elif RCollisions.pointInRect((x, y), self.robotUpRect):
			self.robot()
			return True
			
		x = 800 - x
		y = 600 - y			
				
		for robot in self.robots:
			clicked = RCollisions.pointInRect((x, y), robot.rect)
			if clicked:
				if button == mouse.LEFT:
					if modifiers & key.MOD_SHIFT or modifiers & key.MOD_SHIFT:
						if robot in self.selectedRobots:
							self.selectedRobots.remove(robot)
						else:
							self.selectedRobots.append(robot)
					else:
						self.selectedRobots = [robot]
					break
				elif button == mouse.RIGHT:
					try:
						self.selectedRobots.remove(robot)
						break
					except ValueError:
						pass
		else:
			self.selectedRobots = []
	
	def press(self, symbol, modifiers):
		if symbol == key.G:
			self.gun()
		elif symbol == key.R:
			self.range()
		elif symbol == key.H:
			self.healthUp()
		elif symbol == key.S:
			self.speedUp()
		elif symbol in (key.EQUAL, key.NUM_ADD):
			self.plus()
		elif symbol in (key.MINUS, key.NUM_SUBTRACT):
			self.minus()
		elif symbol == key.N:
			self.robot()
	
	def update(self, keys):
		self.score += .01
		delList = []
		delList2 = []
		for badDude in self.badDudes:
			update = badDude.update()
			if update == BADDUDE_KILLED:
				delList.append(badDude)
			elif update == BADDUDE_ESCAPED:
				delList2.append(badDude)
		for badDude in delList:
			self.score += 10 * 1.1 ** self.waves
			self.gold += 5 * 1.1 ** self.waves
			self.badDudes.remove(badDude)
		for badDude in delList2:
			self.lives -= 1
			self.badDudes.remove(badDude)	
	
		delList = []
		for bullet in self.bullets:
			if bullet.update():
				delList.append(bullet)
		for bullet in delList:
			self.bullets.remove(bullet)			
			
		delList = []
		selectedDelList = []
		for robot in self.robots:
			if robot in self.selectedRobots:
				if robot.update(keys[key.LEFT] or keys[key.A], keys[key.RIGHT] or keys[key.D], keys[key.DOWN] or keys[key.S]):
					selectedDelList.append(robot)
					delList.append(robot)
			else:
				if robot.update(False, False, False):
					delList.append(robot)
					
		for robot in delList:
			self.robots.remove(robot)
		for robot in selectedDelList:
			self.selectedRobots.remove(robot)
						   
		self.ticksUntilNextWave -= 1
		self.distanceMoved += self.speed
						   
		if self.ticksUntilNextWave <= 0:	
			self.ticksUntilNextWave += TICKS_BETWEEN_WAVES
			for i in range(7):
				if self.waves % NUM_BADDUDES == 0:
					self.badDudes.append(RBadDudes.Tank(self, ((80 + random.randrange(41)) + 100 * i, 600 + random.randrange(100) + self.distanceMoved), self.waves))
				elif self.waves % NUM_BADDUDES == 1:
					self.badDudes.append(RBadDudes.Fighter(self, ((80 + random.randrange(41)) + 100 * i, 600 + random.randrange(100) + self.distanceMoved), self.waves))

			self.waves += 1
			self.distanceMoved = 0
		self.scrollY -= self.speed
		
		if self.lives <= 0:
			return self.score
		return False
		
	def draw(self):
		for i in range(0, 800 + 256, 256):
			for j in range(0, 600 + 256, 256):
				self.groundTex.blit(i, j - self.scrollY % 256)
						
		for robot in self.robots:
			robot.draw(robot in self.selectedRobots)			
			
		for badDude in self.badDudes:
			badDude.draw()

		for bullet in self.bullets:
			bullet.draw()
			
		glPushMatrix()
		glTranslatef(400, 300, 0)
		glRotatef(180, 0, 0, 1)
		glTranslatef(-400, -300, 0)
		#GUI
		self.plusTex.blit(self.plusRect.left, self.plusRect.bottom)
		self.minusTex.blit(self.minusRect.left, self.minusRect.bottom)
		
		self.rangeUpTex.blit(self.rangeUpRect.left, self.rangeUpRect.bottom)
		texMid = self.rangeUpRect.left + (self.rangeUpRect.right - self.rangeUpRect.left) / 2
		rangeCost = self.getRangeCost()
		text = font.Text(self.font, ':%i'%rangeCost)
		if self.gold >= rangeCost and len(self.selectedRobots):
			text.color = (0, 0, 0, 1)
		else:
			text.color = (1, 0, 0, 1)
			self.disabledButtonTex.blit(self.rangeUpRect.left, self.rangeUpRect.bottom)
		totalWidth = text.width + self.goldTex.width
		self.goldTex.blit(texMid - totalWidth / 2, self.rangeUpRect.bottom - 16)
		text.x = texMid - totalWidth / 2 + self.goldTex.width
		text.y = self.rangeUpRect.bottom - 16
		text.draw()
		
		self.gunUpTex.blit(self.gunUpRect.left, self.gunUpRect.bottom)
		texMid = self.gunUpRect.left + (self.gunUpRect.right - self.gunUpRect.left) / 2
		gunCost = self.getGunCost()
		text = font.Text(self.font, ':%i'%gunCost)
		bAllowed = True
		for robot in self.selectedRobots:
			if not robot.canGunUp():
				bAllowed = False
		if self.gold >= gunCost and bAllowed and len(self.selectedRobots):
			text.color = (0, 0, 0, 1)
		else:
			text.color = (1, 0, 0, 1)
			self.disabledButtonTex.blit(self.gunUpRect.left, self.gunUpRect.bottom)
		totalWidth = text.width + self.goldTex.width
		self.goldTex.blit(texMid - totalWidth / 2, self.gunUpRect.bottom - 16)
		text.x = texMid - totalWidth / 2 + self.goldTex.width
		text.y = self.gunUpRect.bottom - 16
		text.draw()		
		
		self.speedUpTex.blit(self.speedUpRect.left, self.speedUpRect.bottom)
		speedCost = self.getSpeedCost()
		texMid = self.speedUpRect.left + (self.speedUpRect.right - self.speedUpRect.left) / 2
		text = font.Text(self.font, ':%i'%speedCost)
		if self.gold >= speedCost and len(self.selectedRobots):
			text.color = (0, 0, 0, 1)
		else:
			text.color = (1, 0, 0, 1)
			self.disabledButtonTex.blit(self.speedUpRect.left, self.speedUpRect.bottom)
		totalWidth = text.width + self.goldTex.width
		self.goldTex.blit(texMid - totalWidth / 2, self.speedUpRect.bottom - 16)
		text.x = texMid - totalWidth / 2 + self.goldTex.width
		text.y = self.speedUpRect.bottom - 16
		text.draw()		
		
		self.healthUpTex.blit(self.healthUpRect.left, self.healthUpRect.bottom)
		healthCost = self.getHealthCost()
		texMid = self.healthUpRect.left + (self.healthUpRect.right - self.healthUpRect.left) / 2
		text = font.Text(self.font, ':%i'%healthCost)
		if self.gold >= healthCost and len(self.selectedRobots):
			text.color = (0, 0, 0, 1)
		else:
			text.color = (1, 0, 0, 1)
			self.disabledButtonTex.blit(self.speedUpRect.left, self.healthUpRect.bottom)
		
		totalWidth = text.width + self.goldTex.width
		self.goldTex.blit(texMid - totalWidth / 2, self.healthUpRect.bottom - 16)
		text.x = texMid - totalWidth / 2 + self.goldTex.width
		text.y = self.healthUpRect.bottom - 16
		text.draw()
		
		self.robotUpTex.blit(self.robotUpRect.left, self.robotUpRect.bottom)
		robotCost = self.getRobotCost()
		texMid = self.robotUpRect.left + (self.robotUpRect.right - self.robotUpRect.left) / 2
		text = font.Text(self.font, ':%i'%robotCost)
		if self.gold >= robotCost:
			text.color = (0, 0, 0, 1)
		else:
			text.color = (1, 0, 0, 1)
			self.disabledButtonTex.blit(self.robotUpRect.left, self.robotUpRect.bottom)
			
		totalWidth = text.width + self.goldTex.width
		self.goldTex.blit(texMid - totalWidth / 2, self.robotUpRect.bottom - 16)
		text.x = texMid - totalWidth / 2 + self.goldTex.width
		text.y = self.robotUpRect.bottom - 16
		text.draw()		
		
		barHeight = (self.speedBarRect.top - self.speedBarRect.bottom - 15) * self.speed / self.maxSpeed
		
		glColor4f(1.0, 0.0, 0.0, 0.9)
		glBegin(GL_QUADS)
		glVertex2f(self.speedBarRect.left + 10, self.speedBarRect.bottom + 10)
		glVertex2f(self.speedBarRect.right - 10, self.speedBarRect.bottom + 10)
		glVertex2f(self.speedBarRect.right - 10, self.speedBarRect.bottom + 10 + barHeight)
		glVertex2f(self.speedBarRect.left + 10, self.speedBarRect.bottom + 10 + barHeight)
		glEnd()
		self.speedBarTex.blit(self.speedBarRect.left, self.speedBarRect.bottom)
		
		
		#
		self.topPanelTex.blit(530, 600 - 38)
		
		text = font.Text(self.font, ':%i'%self.lives)
		text.color = (0, 0, 0, 1)
		self.lifeTex.blit(540, 600 - 30)
		text.x = 540 + 20
		text.y = 600 - 30
		text.draw()		
		
		text = font.Text(self.font, ':%i'%self.gold)
		text.color = (0, 0, 0, 1)
		self.goldTex.blit(650, 600 - 30)
		text.x = 650 + 20
		text.y = 600 - 30
		text.draw()
		
		text = font.Text(self.font, '%i'%int(self.score), halign=font.Text.CENTER)
		text.color = (0, 0, 0, 1)
		text.x = 623
		text.y = 600 - 30
		text.draw()		
		
		glPopMatrix()
	